% plato.inc

% platonic solids

% (matrix3d.inc) run
% (polyhedron.inc) run

% for each one, a set of vectors 
% alpha[0], alpha[1], alpha[2]
% varpi[0], varpi[1], varpi[2]
% alpha[0] = [1 0 0]
% alpha[1] = [x y 0] with y > 0
% alpha[2] = [x y z] with z > 0
% the "base face" is parallel to xy plane with z > 0
% centred at varpi[2]
% the rest are obtained from this by words in invshortlex
% each figure is an array of faces
% [face normal]
% where face = array of vertices

% - common ----------------------------------------------------

% applies right to left

% v word alpha
% v => s_alpha[w1] s_alpha[w2] ... s_alpha[wn] v 

/platodict 32 dict def

platodict begin

/reflect-word {
6 dict begin
/alpha exch def
/word exch def
/v exch def
v 
word length 1 sub -1 0 {
/i exch def
/k word i get def
alpha k get reflect-3d
} for
end
} def

% v[] word alpha
% applies previous routine to each v[i]

/polygon-reflect-word {
6 dict begin
/alpha exch def
/word exch def
/v exch def
/N v length def
[
0 1 N 1 sub {
/i exch def
v i get word alpha reflect-word
} for
]
end
} def

/poly-rotate {
alpha 0 get reflect-3d
alpha 1 get reflect-3d
} def

% - original data ----------------------------------------------

/tetrahedron-n 	[3 2 3] def
/cube-n 		[3 2 4] def
/octahedron-n 	[4 2 3] def
/dodecahedron-n [3 2 5] def
/icosahedron-n 	[5 2 3] def

/tetrahedron-word [
     []
    [2] 
  [1 2]
[0 1 2]
] def

/cube-word [
         []
        [2]
      [1 2]
    [0 1 2]
  [1 0 1 2]
[2 1 0 1 2] 
] def

/octahedron-word [
           []
          [2]
        [1 2] 
      [0 1 2] 
    [2 0 1 2] 
  [1 2 0 1 2] 
[2 1 2 0 1 2] 
      [2 1 2] 
] def

/dodecahedron-word [
                   []
                  [2] 
                [1 2] 
              [0 1 2] 
            [1 0 1 2] 
          [0 1 0 1 2] 
        [2 0 1 0 1 2] 
      [1 2 0 1 0 1 2] 
    [0 1 2 0 1 0 1 2] 
  [1 0 1 2 0 1 0 1 2] 
[2 1 0 1 2 0 1 0 1 2]
          [2 1 0 1 2] 
] def

/icosahedron-word [ 
                       []
                      [2] 
                    [1 2] 
                  [0 1 2] 
                [2 0 1 2] 
              [1 2 0 1 2] 
            [0 1 2 0 1 2] 
          [2 0 1 2 0 1 2] 
        [1 2 0 1 2 0 1 2] 
      [0 1 2 0 1 2 0 1 2] 
    [2 0 1 2 0 1 2 0 1 2] 
  [1 2 0 1 2 0 1 2 0 1 2] 
[2 1 2 0 1 2 0 1 2 0 1 2]
      [2 1 2 0 1 2 0 1 2] 
            [2 1 2 0 1 2] 
          [1 2 1 2 0 1 2] 
        [0 1 2 1 2 0 1 2] 
                  [2 1 2] 
                [1 2 1 2]
              [0 1 2 1 2] 
] def

% - tetrahedra -------------------------------------------------

/tetrahedron-alpha [
8 dict begin
/x2 180 tetrahedron-n 2 get div cos neg def
/y2 180 tetrahedron-n 2 get div sin def
/x3 0 def
/y3 180 tetrahedron-n 0 get div cos y2 neg div def
/z3 1 y3 dup mul sub sqrt def
[1 0 0]
[x2 y2 0]
[0 y3 z3]
end
] 
def

/tetrahedron-varpi
1 dict begin
/eta   
tetrahedron-alpha 
inverse-3d transpose-3d
def
[
eta 0 get normalize-3d
eta 1 get normalize-3d eta 0 get eta 1 get angle-between-3d cos vector-scale-3d
eta 2 get normalize-3d eta 0 get eta 2 get angle-between-3d cos vector-scale-3d
]
end
def

% - cubes ------------------------------------------------------

/cube-alpha [
8 dict begin
/x2 180 cube-n 2 get div cos neg def
/y2 180 cube-n 2 get div sin def
/x3 0 def
/y3 180 cube-n 0 get div cos y2 neg div def
/z3 1 y3 dup mul sub sqrt def
[1 0 0]
[x2 y2 0]
[0 y3 z3]
end
] 
def

/cube-varpi
1 dict begin
/eta   
cube-alpha 
inverse-3d transpose-3d
def
[
eta 0 get normalize-3d
eta 1 get normalize-3d eta 0 get eta 1 get angle-between-3d cos vector-scale-3d
eta 2 get normalize-3d eta 0 get eta 2 get angle-between-3d cos vector-scale-3d
]
end
def

% - octahedra ------------------------------------------------------

/octahedron-alpha [
8 dict begin
/x2 180 octahedron-n 2 get div cos neg def
/y2 180 octahedron-n 2 get div sin def
/x3 0 def
/y3 180 octahedron-n 0 get div cos y2 neg div def
/z3 1 y3 dup mul sub sqrt def
[1 0 0]
[x2 y2 0]
[0 y3 z3]
end
] 
def

/octahedron-varpi
1 dict begin
/eta   
octahedron-alpha 
inverse-3d transpose-3d
def
[
eta 0 get normalize-3d
eta 1 get normalize-3d eta 0 get eta 1 get angle-between-3d cos vector-scale-3d
eta 2 get normalize-3d eta 0 get eta 2 get angle-between-3d cos vector-scale-3d
]
end
def

% - dodecahedron ------------------------------------------------------

/dodecahedron-alpha [
8 dict begin
/x2 180 dodecahedron-n 2 get div cos neg def
/y2 180 dodecahedron-n 2 get div sin def
/x3 0 def
/y3 180 dodecahedron-n 0 get div cos y2 neg div def
/z3 1 y3 dup mul sub sqrt def
[1 0 0]
[x2 y2 0]
[0 y3 z3]
end
] 
def

/dodecahedron-varpi
1 dict begin
/eta   
dodecahedron-alpha 
inverse-3d transpose-3d
def
[
eta 0 get normalize-3d
eta 1 get normalize-3d eta 0 get eta 1 get angle-between-3d cos vector-scale-3d
eta 2 get normalize-3d eta 0 get eta 2 get angle-between-3d cos vector-scale-3d
]
end
def

% - icosahedra ------------------------------------------------------

/icosahedron-alpha [
8 dict begin
/x2 180 icosahedron-n 2 get div cos neg def
/y2 180 icosahedron-n 2 get div sin def
/x3 0 def
/y3 180 icosahedron-n 0 get div cos y2 neg div def
/z3 1 y3 dup mul sub sqrt def
[1 0 0]
[x2 y2 0]
[0 y3 z3]
end
] 
def

/icosahedron-varpi
1 dict begin
/eta   
icosahedron-alpha 
inverse-3d transpose-3d
def
[
eta 0 get normalize-3d
eta 1 get normalize-3d eta 0 get eta 1 get angle-between-3d cos vector-scale-3d
eta 2 get normalize-3d eta 0 get eta 2 get angle-between-3d cos vector-scale-3d
]
end
def

end % platodict

% ---------------------------------------------------------------------------
% ---------------------------------------------------------------------------

/tetrahedron { 
platodict begin
8 dict begin
/facecount tetrahedron-word length def
/alpha tetrahedron-alpha def

% f0 obtained by rotating varpi[0] 3 times

/f0 

[ 
tetrahedron-varpi 0 get 

tetrahedron-varpi 0 get  
poly-rotate

tetrahedron-varpi 0 get 
poly-rotate
poly-rotate

tetrahedron-varpi 0 get 
]
def

[
[f0 [0 0 1]]

1 1 facecount 1 sub {
/i exch def
/w tetrahedron-word i get def
[
f0 w alpha polygon-reflect-word
% w length 2 mod 0 eq
% {
% [0 0 1]
% } {
% [0 0 1]
% } ifelse
[0 0 1] w alpha reflect-word
]
} for
]
end
end }
def

/cube {
platodict begin
8 dict begin
/facecount cube-word length def
/alpha cube-alpha def

% f0 obtained by rotating varpi[0] 4 times

/f0 

[ 
cube-varpi 0 get 

cube-varpi 0 get 
poly-rotate

cube-varpi 0 get 
poly-rotate
poly-rotate

cube-varpi 0 get 
poly-rotate
poly-rotate
poly-rotate

cube-varpi 0 get 
]
def

[
[f0 [0 0 1]]

1 1 facecount 1 sub {
/i exch def
/w cube-word i get def
[
f0 w alpha polygon-reflect-word
[0 0 1] w alpha reflect-word
]
} for
]
end 
end }
def

/octahedron {
platodict begin
8 dict begin
/facecount octahedron-word length def
/alpha octahedron-alpha def

% f0 obtained by rotating varpi[0] 4 times

/f0 

[ 
octahedron-varpi 0 get 

octahedron-varpi 0 get 
poly-rotate

octahedron-varpi 0 get 
poly-rotate
poly-rotate

octahedron-varpi 0 get 
]
def

[
[f0 [0 0 1]]

1 1 facecount 1 sub {
/i exch def
/w octahedron-word i get def
[
f0 w alpha polygon-reflect-word
[0 0 1] w alpha reflect-word
]
} for
]
end 
end }
def

/dodecahedron {
platodict begin
8 dict begin
/facecount dodecahedron-word length def
/alpha dodecahedron-alpha def

% f0 obtained by rotating varpi[0] 4 times

/f0 

[ 
dodecahedron-varpi 0 get 

dodecahedron-varpi 0 get 
poly-rotate

dodecahedron-varpi 0 get 
poly-rotate
poly-rotate

dodecahedron-varpi 0 get 
poly-rotate
poly-rotate
poly-rotate

dodecahedron-varpi 0 get 
poly-rotate
poly-rotate
poly-rotate
poly-rotate

dodecahedron-varpi 0 get 
]
def

[
[f0 [0 0 1]]

1 1 facecount 1 sub {
/i exch def
/w dodecahedron-word i get def
[
f0 w alpha polygon-reflect-word
[0 0 1] w alpha reflect-word
]
} for
]
end 
end }
def

/icosahedron {
platodict begin
8 dict begin
/facecount icosahedron-word length def
/alpha icosahedron-alpha def

% f0 obtained by rotating varpi[0] 3 times

/f0 

[ 
icosahedron-varpi 0 get 

icosahedron-varpi 0 get 
poly-rotate

icosahedron-varpi 0 get 
poly-rotate
poly-rotate

icosahedron-varpi 0 get 
]
def

[
[f0 [0 0 1]]

1 1 facecount 1 sub {
/i exch def
/w icosahedron-word i get def
[
f0 w alpha polygon-reflect-word
[0 0 1] w alpha reflect-word
]
} for
]
end 
end }
def

% --------------------------------------------------------------

