% surfaces.inc

(../../ps/const.inc) run

% a surface is an array of faces [polygon normal]

% ico-face = array of
% [triangle normal]
% triangle =array of [. . .]
% so the j-th vertex of the i-th face is
% i get (face) 0 get (triangle) j get (vertex) 

/icosahedron-face 

8 dict begin

/x 5 sqrt 1 add 2 div def
/r x dup mul 1 add sqrt def
/a x r div def
/b 1 r div def

/t [
[[a b 0] [0 a b] [b 0 a]] 						% 11 31 21
[[a b 0] [b 0 a neg] [0 a b neg]] 				% 11 22 33
[[a b neg 0] [b 0 a] [0 a neg  b]] 				% 13 21 32
[[a b neg 0] [0 a neg b neg] [b 0 a neg]] 		% 13 34 22

[[a neg  b 0] [b neg 0 a] [0 a b]] 				% 12 23 31
[[a neg  b 0] [0 a b neg] [b neg 0 a neg]] 		% 12 33 24
[[a neg b neg 0] [0 a neg  b] [b neg 0 a]] 		% 14 32 23
[[a neg b neg 0] [b neg 0 a neg] [0 a neg b neg]] % 14 24 34

[[a b 0] [0 a b neg] [0 a b]] 					% 11 33 31
[[a neg  b 0] [0 a b] [0 a b neg]] 				% 12 31 33
[[a b neg 0] [0 a neg  b] [0 a neg b neg]] 		% 13 32 34
[[a neg b neg 0] [0 a neg b neg] [0 a neg  b]] 	% 14 34 32

[[b 0 a] [a b neg 0] [a b 0]] 					% 21 13 11
[[b 0 a neg] [a b 0] [a b neg 0]] 				% 22 11 13
[[b neg 0 a] [a neg  b 0] [a neg b neg 0]] 		% 23 12 14
[[b neg 0 a neg] [a neg b neg 0] [a neg  b 0]] 	% 24 b4 b2

[[0 a b] [b neg 0 a] [b 0 a]] 					% 31 23 21
[[0 a neg  b] [b 0 a] [b neg 0 a]] 				% 32 21 23
[[0 a b neg] [b 0 a neg] [b neg 0 a neg]] 		% 33 22 24
[[0 a neg b neg] [b neg 0 a neg] [b 0 a neg]] 	% 34 24 22
] def

[
0 1 19 {
/i exch def
[t i get 
dup /F exch def
F 1 get F 0 get vector-sub-3d
F 2 get F 0 get vector-sub-3d
cross-product
normalize-3d
]
} for
]

end

def

% -----------------------------------------------

% spherical-subdivision 
% [[P Q R] N] => [4 subtriangles + normals]

/spherical-subdivision {
8 dict begin
aload pop
/N exch def
/T exch def
/R T 2 get def
/Q T 1 get def
/P T 0 get def
/PQ [
P 0 get Q 0 get add 0.5 mul
P 1 get Q 1 get add 0.5 mul
P 2 get Q 2 get add 0.5 mul
]
normalize-3d 
def
/PR [
P 0 get R 0 get add 0.5 mul
P 1 get R 1 get add 0.5 mul
P 2 get R 2 get add 0.5 mul
]
normalize-3d 
def
/QR [
R 0 get Q 0 get add 0.5 mul
R 1 get Q 1 get add 0.5 mul
R 2 get Q 2 get add 0.5 mul
]
normalize-3d 
def

[[ P PQ PR] 
PQ P vector-sub-3d
PR P vector-sub-3d 
cross-product normalize-3d ]
[[ Q QR PQ] 
QR Q vector-sub-3d
PQ Q vector-sub-3d
cross-product normalize-3d ]
[[ R PR QR] 
PR R vector-sub-3d
QR R vector-sub-3d
cross-product normalize-3d ]
[[PQ QR PR] 
QR PQ vector-sub-3d
PR PQ vector-sub-3d
cross-product normalize-3d ]

end
} def

% - sphere -------------------------------------------------

% [x y z R] N => sphere of 20 x 4^N triangles based on icosahedron, radius R

/sphere {
8 dict begin
/N exch def
aload pop
/R exch def
/z exch def
/y exch def
/x exch def
/sd /spherical-subdivision cvx def

/PL [
0 1 19 {
/i exch def
icosahedron-face i get
} for
] def

N { 

/QL PL length 4 mul array def
/k 0 def
PL {
sd
QL k 3 add 3 2 roll put
QL k 2 add 3 2 roll put
QL k 1 add 3 2 roll put
QL k       3 2 roll put
/k k 4 add def
} forall

/PL QL def

} repeat

PL {
/F exch def
F 0 get 0
2 copy    get R vector-scale-3d [x y z] vector-add-3d
put
F 0 get 1
2 copy    get R vector-scale-3d [x y z] vector-add-3d
put
F 0 get 2
2 copy    get R vector-scale-3d [x y z] vector-add-3d
put
} forall

PL
end
} def

% in spherical coordinates 

% [R] a0 a1 b0 b1 M N

% [R cos a cos b R cos a sin b R sin a]

/legendre-sphere {
24 dict begin
/N exch def
/M exch def
/b1 exch  def
/b0 exch  def
/a1 exch  def
/a0 exch  def
aload pop /R exch def
/ainc a1 a0 sub M div def
/binc b1 b0 sub N div def
/a a0 def

[
M {
  /b b0 def
  /nexta a ainc add def
  N {
    % MN rectangles altogether
    /nextb b binc add def
	/ca a cos def
	/cb b cos def
	/sa a sin def
	/sb b sin def
    [
      [
        [ca cb mul R mul ca sb mul R mul sa R mul] 
        [ca nextb cos mul R mul ca nextb sin mul R mul sa R mul] 
        [nexta cos nextb cos mul R mul nexta cos nextb sin mul R mul nexta sin R mul] 
        [nexta cos cb mul R mul nexta cos sb mul R mul nexta sin R mul] 
        [ca cb mul R mul ca sb mul R mul sa R mul] 
      ]
      ca 0 eq {
		a 0 gt {
		  [0 0 1]
		} {
		  [0 0 -1]
		  } ifelse
		} {
		  [ca nextb cos mul R mul ca nextb sin mul R mul sa R mul]
          [ca cb mul R mul ca sb mul R mul sa R mul]
		  vector-sub-3d
		  [nexta cos cb mul R mul nexta cos sb mul R mul nexta sin R mul] 
          [ca cb mul R mul ca sb mul R mul sa R mul] 
		  vector-sub-3d 
		  cross-product normalize-3d 
		} ifelse
    ]
    /b nextb def
  } repeat
  /a nexta def
} repeat
]
end
} def

% - cone --------------------------------------------------

% x in [-1, 1] => shade of gray to make it
% [a b] x => (1/2)((b-a)x + (b+a))

/linear-shade { 3 dict begin
/x exch def
aload pop 
/b exch def
/a exch def
b a sub x mul b add a add 0.5 mul end } def

% a cone is a union of triangular strips
% [half vertex-angle + height] + initial angle + final angle + N 
% ht can be negative

/cone {
8 dict begin
/N exch def
/tf exch def
/t exch def
aload pop
/ht exch def
/a exch def

/r a sin a cos div ht abs mul def
% r is always positive, so (x, y) = projection onto (x,y) plane
/dt tf t sub N div def
/dt2 dt 0.5 mul def

% array of [[[...][...][...]] [...]]
[

N {
[
[[0 0 0][t dt add cos r mul
		 t dt add sin r mul
		 ht ][t cos r mul
			 t sin r mul 
			ht ]]
[ t dt2 add cos 
t dt2 add sin 
r ht div neg ] normalize-3d
]
/t dt t add def
} repeat

]
end
} def

% a frustrum is a union of triangular strips
% [half vertex-angle + height0 + height1] + initial angle + final angle + N 
% ht can be negative

/frustrum {
16 dict begin
/N exch def
/tf exch def
/t exch def
aload pop
/ht1 exch def
/ht0 exch def
/a exch def

/r0 a sin a cos div ht0 abs mul def
/r1 a sin a cos div ht1 abs mul def
% r is always positive, so (x, y) = projection onto (x,y) plane
/dt tf t sub N div def
/dt2 dt 0.5 mul def

% array of [[[...][...][...]] [...]]
[

N {
[
[
[
  t dt add cos r0 mul
  t dt add sin r0 mul
  ht0 
]
[
  t cos r0 mul
  t sin r0 mul 
  ht0 
]
[
  t cos r1 mul
  t sin r1 mul
  ht1 
][
  t dt add cos r1 mul
  t dt add sin r1 mul 
  ht1 
]
]
[ t dt2 add cos 
t dt2 add sin 
r0 ht0 div neg ] normalize-3d
]
/t dt t add def
} repeat

]
end
} def

% - cylinder ---------------------------------------------

% a cylinder is a union of triangular strips
% [R + height0 + height1] + initial angle + final angle + N 
% ht can be negative

/cylinder {
16 dict begin
/N exch def
/tf exch def
/t exch def
aload pop
/ht1 exch def
/ht0 exch def
/R exch def

% r is always positive, so (x, y) = projection onto (x,y) plane
/dt tf t sub N div def
/dt2 dt 0.5 mul def

% array of [[[...][...][...]] [...]]
[

N {
[
[
[
  t dt add cos R mul
  t dt add sin R mul
  ht0 
]
[
  t cos R mul
  t sin R mul 
  ht0 
]
[
  t cos R mul
  t sin R mul
  ht1 
][
  t dt add cos R mul
  t dt add sin R mul 
  ht1 
]
]
[ t dt2 add cos 
t dt2 add sin 
0 ] normalize-3d
]
/t dt t add def
} repeat

]
end
} def

% along z-axis

% R L N

/old-cylinder {
8 dict begin
/N exch def
/L exch def
/R exch def
/T 360 N div def
/A 0 def
% make an array of vertices of regular polygon od radius R
% poly[i] = [R cos iT, R sin iT]
/poly [
N {
	[A cos R mul A sin R mul]
	/A A T add def
} repeat
] def

% end at z=0
[

% a face = [ array of vertices, normal ]
% end
[
[
0 1 N 1 sub {/i exch def
[
 	poly i get aload pop
	0
]
} for
]
[0 0 -1]
]

% other end
[
[
0 1 N 1 sub {/i exch def
[
 	poly i get aload pop
	L
]
} for
]
[0 0 1]
]

% the N sides
0 1 N 1 sub {
/i exch def
/iplus i 1 add def
iplus N eq {
/iplus 0 def
} if
[
[
[poly i get aload pop 0]
[poly iplus get aload pop 0]
[poly iplus get aload pop L]
[poly i get aload pop L]
[poly i get aload pop 0]
]
[
	i 0.5 add T mul cos 
	i 0.5 add T mul sin 
	0 
]
]
} for

]
end
} def

% -------------------------------------------
